@@ -6,7 +6,7 @@ use Carp qw/croak/;
 use Class::XSAccessor::Heavy;
 use XSLoader;
 
-our $VERSION = '1.07_04';
+our $VERSION = '1.12_01';
 
 XSLoader::load('Class::XSAccessor', $VERSION);
 
@@ -34,22 +34,26 @@ sub import {
   $caller_pkg = $opts{class} if defined $opts{class};
 
   # TODO: Refactor. Move more duplicated code to ::Heavy
-  my $read_subs      = _make_hash($opts{getters} || {});
-  my $set_subs       = _make_hash($opts{setters} || {});
-  my $acc_subs       = _make_hash($opts{accessors} || {});
-  my $lvacc_subs     = _make_hash($opts{lvalue_accessors} || {});
-  my $pred_subs      = _make_hash($opts{predicates} || {});
-  my $test_subs      = _make_hash($opts{__tests__} || {});
-  my $construct_subs = $opts{constructors} || [defined($opts{constructor}) ? $opts{constructor} : ()];
-  my $true_subs      = $opts{true} || [];
-  my $false_subs     = $opts{false} || [];
+  my $read_subs        = _make_hash($opts{getters} || {});
+  my $set_subs         = _make_hash($opts{setters} || {});
+  my $acc_subs         = _make_hash($opts{accessors} || {});
+  my $lvacc_subs       = _make_hash($opts{lvalue_accessors} || {});
+  my $pred_subs        = _make_hash($opts{predicates} || {});
+  my $test_subs        = _make_hash($opts{__tests__} || {});
+  my $cached_read_subs = _make_hash($opts{cached_getters} || {});
+  my $cached_acc_subs  = _make_hash($opts{cached_accessors} || {});
+  my $construct_subs   = $opts{constructors} || [defined($opts{constructor}) ? $opts{constructor} : ()];
+  my $true_subs        = $opts{true} || [];
+  my $false_subs       = $opts{false} || [];
 
   foreach my $subtype ( ["getter", $read_subs],
                         ["setter", $set_subs],
                         ["accessor", $acc_subs],
                         ["lvalue_accessor", $lvacc_subs],
                         ["test", $test_subs],
-                        ["pred_subs", $pred_subs] )
+                        ["predicate", $pred_subs],
+                        ["cached_getter", $cached_read_subs],
+                        ["cached_accessor", $cached_acc_subs], )
   {
     my $subs = $subtype->[1];
     foreach my $subname (keys %$subs) {
@@ -82,6 +86,9 @@ sub _generate_method {
   if ($type eq 'getter') {
     newxs_getter($subname, $hashkey);
   }
+  elsif ($type eq 'cached_getter') {
+    newxs_cached_getter($subname, $hashkey);
+  }
   elsif ($type eq 'lvalue_accessor') {
     newxs_lvalue_accessor($subname, $hashkey);
   }
@@ -103,6 +110,9 @@ sub _generate_method {
   elsif ($type eq 'test') {
     newxs_test($subname, $hashkey);
   }
+  elsif ($type eq 'cached_accessor') {
+    newxs_cached_accessor($subname, $hashkey, 0); # no chained variant available
+  }
   else {
     newxs_accessor($subname, $hashkey, $opts->{chained}||0);
   }
@@ -118,6 +128,8 @@ Class::XSAccessor - Generate fast XS accessors without runtime compilation
 
 =head1 SYNOPSIS
 
+  # This synopsis just shows all accessor types available,
+  # in reality, you'd typically only use one or few of them.
   package MyClass;
   use Class::XSAccessor
     replace     => 1,   # Replace existing methods (if any)
@@ -141,6 +153,12 @@ Class::XSAccessor - Generate fast XS accessors without runtime compilation
     lvalue_accessors => { # see below
       baz => 'baz', # ...
     },
+    cached_getters => { # see below
+      get_blargl => 'blargl', # ...
+    },
+    cached_accessors => { # see below
+      blargl => 'blargl', # ...
+    },
     true  => [ 'is_token', 'is_whitespace' ],
     false => [ 'significant' ];
   
@@ -184,11 +202,11 @@ That means they can be called on objects and classes but will not
 clone objects entirely. Parameters to C<new()> are added to the
 object.
 
-The XS accessor methods are between 3 and 4 times faster than typical
+The XS accessor methods are between 3 and 5 times faster than typical
 pure-Perl accessors in some simple benchmarking.
 The lower factor applies to the potentially slightly obscure
 C<sub set_foo_pp {$_[0]-E<gt>{foo} = $_[1]}>, so if you usually
-write clear code, a factor of 3.5 speed-up is a good estimate.
+write clear code, at least a factor of 3.5 speed-up is a good estimate.
 If in doubt, do your own benchmarking!
 
 The method names may be fully qualified. The example in the synopsis could
@@ -241,6 +259,56 @@ in the same C<use Class::XSAccessor ...> statement.
 By default, the accessors are generated in the calling class. The
 the C<class> option allows the target class to be specified.
 
+=head1 CACHED ACCESSORS
+
+It takes a couple of words to explain what I mean by I<cached getter>
+and I<cached accessor>. But most of that can be done away with by
+telling you what Perl code the I<cached getters> implement more efficiently:
+
+  sub get_foo {
+    my $self = shift;
+    if (not exists($self->{foo})) {
+      $self->{foo} = $self->_get("foo");
+    }
+    return $self->{foo};
+  }
+
+Similarly, I<cached accessors> implement the following:
+
+  sub foo {
+    my $self = shift;
+    if (@_) {
+      $self->_set("foo", $_[0]);
+      return $_[0];
+    }
+    else {
+      if (not exists($self->{foo})) {
+        $self->{foo} = $self->_get("foo");
+      }
+      return $self->{foo};
+    }
+  }
+
+This means that both the read-only accessor/getter C<get_foo> and the
+read-write accessor C<foo> will return the value stored in the C<$self>
+hash as the "foo" entry if it exists (and in the rw case, if called
+without additional arguments). If that hash entry does not exist, both
+will call the C<_get> method (to be implemented by you) with the hash key
+name "foo" as argument and set the hash entry "foo" to whatever C<_get>
+returns.
+
+The rw-accessor will call the C<_set> method (again, to be implemented by
+you) if you provide additional arguments.
+
+This implements a form of lazy evaluation commonly found in ORMs.
+The important feature here is that the most common case -- the corresponding
+hash entry is to be fetched from the very hash -- will be very fast. Only
+if it doesn't exist, more expensive calculations will be made (such as
+fetching data from a database). Note that if either C<_get> or C<_set>
+needs to be invoked, the overall performance will be similar
+(slightly faster on my machine in a trivial benchmark) 
+to the pure-Perl implementation since calling into Perl from C is very slow.
+
 =head1 LVALUES
 
 Support for lvalue accessors via the keyword C<lvalue_accessors>
@@ -299,7 +367,7 @@ chocolateboy E<lt>chocolate@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2008-2010 by Steffen Mueller
+Copyright (C) 2008, 2009, 2010, 2011 by Steffen Mueller
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8 or,
